"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020-2022 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:vdb' }"

"{ NameSpace: Smalltalk }"

VDBAbstractTreeApplication subclass:#VDBInstructionListApplication
	instanceVariableNames:'instructionListHolder instructionBasicBlocks
		instructionBasicBlocksAnalyzerJob selectedInstructionHolder
		selectedInstructionBranchTargetAddress
		selectedInstructionBasicBlock disassemblableHolder frameHolder
		canExecStepOverInstructionHolder canExecStepIntoInstructionHolder
		canExecBackOverInstructionHolder canExecBackIntoInstructionHolder'
	classVariableNames:'InstructionListCache InstructionBasicBlocksCache'
	poolDictionaries:''
	category:'VDB-UI-Others'
!

!VDBInstructionListApplication class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020-2022 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
! !

!VDBInstructionListApplication class methodsFor:'initialization'!

initialize
    "Invoked at system start or when the class is dynamically loaded."

    "/ please change as required (and remove this comment)

    InstructionListCache := CacheDictionary new: 8.
    InstructionBasicBlocksCache := CacheDictionary new: 8.
! !

!VDBInstructionListApplication class methodsFor:'accessing'!

defaultWindowTitle
    ^ self resources string:'Assembly'

    "Created: / 22-06-2018 / 12:25:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication class methodsFor:'menu specs'!

contextMenu
    "This resource specification was automatically generated
     by the MenuEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the MenuEditor may not be able to read the specification."


    "
     MenuEditor new openOnClass:VDBInstructionListApplication andSelector:#contextMenu
     (Menu new fromLiteralArrayEncoding:(VDBInstructionListApplication contextMenu)) startUp
    "

    <resource: #menu>

    ^ 
     #(Menu
        (
         (MenuItem
            label: 'Context Menu Slice'
            isVisible: true
            submenuChannel: contextMenuApplSlice
            isMenuSlice: true
          )
         (MenuItem
            label: 'Item Menu Slice'
            isVisible: true
            submenuChannel: contextMenuItemSlice
            isMenuSlice: true
          )
         (MenuItem
            label: 'Copy Menu Slice'
            isVisible: true
            submenuChannel: contextMenuCopySlice
            isMenuSlice: true
          )
         (MenuItem
            label: '-'
            isVisible: true
          )
         (MenuItem
            label: 'Inspect Menu Slice'
            isVisible: true
            submenuChannel: contextMenuInspectSlice
            isMenuSlice: true
          )
         (MenuItem
            label: '-'
          )
         (MenuItem
            label: 'Pin Menu'
            itemValue: doPinMenuAs:item:
            isVisible: true
            labelImage: (ResourceRetriever VDBIconLibrary pin 'Pin Menu')
            argument: 'Disassembly Actions'
          )
         )
        nil
        nil
      )

    "Modified: / 01-09-2018 / 14:55:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

contextMenuApplSlice
    "This resource specification was automatically generated
     by the MenuEditor of ST/X."

    "Do not manually edit this!! If it is corrupted,
     the MenuEditor may not be able to read the specification."


    "
     MenuEditor new openOnClass:VDBInstructionListApplication andSelector:#contextMenuApplSlice
     (Menu new fromLiteralArrayEncoding:(VDBInstructionListApplication contextMenuApplSlice)) startUp
    "

    <resource: #menu>

    ^ 
     #(Menu
        (
         (MenuItem
            enabled: canShowCurrentHolder
            label: 'Show Current Instruction'
            itemValue: doShowCurrent
            isVisible: true
          )
         (MenuItem
            enabled: canShowBranchTargetHolder
            label: 'Show Branch Traget'
            itemValue: doShowBranchTarget
            isVisible: true
          )
         (MenuItem
            enabled: canShowSelectionHolder
            label: 'Show Selection'
            itemValue: doShowSelection
            isVisible: true
          )
         (MenuItem
            label: '-'
            isVisible: true
          )
         (MenuItem
            enabled: canExecStepOverInstructionHolder         
            label: 'Step Over'
            itemValue: doExecStepOverInstruction
            isVisible: true
            labelImage: (ResourceRetriever VDBIconLibrary actionStepOver6x16 'Step Over One Instruction')
          )
         (MenuItem
            enabled: canExecStepIntoInstructionHolder
            label: 'Step Into'
            itemValue: doExecStepIntoInstruction
            isVisible: true
            labelImage: (ResourceRetriever VDBIconLibrary actionStepInto6x16 'Step Into One Instruction')
          )
         (MenuItem
            label: '-'
            isVisible: true
          )
         (MenuItem
             enabled: canExecBackOverInstructionHolder
            label: 'Back Over'
            itemValue: doExecBackOverInstruction
            isVisible: true
          )
         (MenuItem
            enabled: canExecBackIntoInstructionHolder
            label: 'Back Into'
            itemValue: doExecBackIntoInstruction
            isVisible: true
          )
         (MenuItem
            label: '-'
            isVisible: true
          )
         )
        nil
        nil
      )

    "Modified: / 01-09-2018 / 22:02:42 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication class methodsFor:'plugIn spec'!

aspectSelectors
    "This resource specification was automatically generated
     by the UIPainter of ST/X."

    "Do not manually edit this. If it is corrupted,
     the UIPainter may not be able to read the specification."

    "Return a description of exported aspects;
     these can be connected to aspects of an embedding application
     (if this app is embedded in a subCanvas)."

    ^ #(
        #debuggerHolder
        #disassemblableHolder 
        #frameHolder
      ).

    "Created: / 20-10-2019 / 23:35:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication class methodsFor:'utilities'!

instructions: aCollection title: aString
    ^ self new  
        instructionList: aCollection;
        title: aString;
        yourself

    "Created: / 26-06-2018 / 11:03:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication methodsFor:'accessing'!

disassemblable
    ^ self disassemblableHolder value

    "Created: / 10-06-2019 / 10:06:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

disassemblable: anObject
    self disassemblableHolder value: anObject

    "Created: / 10-06-2019 / 10:06:06 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

frame
    ^ self frameHolder value

    "Created: / 10-06-2019 / 12:44:27 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

frame: aGDBFrame
    ^ self frameHolder value: aGDBFrame

    "Created: / 10-06-2019 / 12:44:34 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

instructionList
    ^ self instructionListHolder value

    "Created: / 07-08-2018 / 13:27:50 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

instructionList: aCollection
    self instructionListHolder value: aCollection

    "Created: / 22-06-2018 / 12:48:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication methodsFor:'aspects'!

backgroundColorFor: aVDBPresenter
    <resource: #style (#'instructionListApplication.currentInstructionBackgroundColor'
                       #'instructionListApplication.branchTargetBackgroundColor'
                       #'instructionListApplication.currentBasicBlockBackgroundColor'
                       #'instructionListApplication.successor1BasicBlockBackgroundColor'
                       #'instructionListApplication.successor2BasicBlockBackgroundColor'
                      )>

    (self frame notNil and:[aVDBPresenter isInstructionPresenter]) ifTrue:[ 
        (frameHolder value notNil and:[frameHolder value address == aVDBPresenter address]) ifTrue:[ 
            ^ self styleSheet colorAt: 'instructionListApplication.currentInstructionBackgroundColor' default:[ Color green ].
        ].
    ].

    selectedInstructionBranchTargetAddress notNil ifTrue:[ 
        aVDBPresenter isInstructionPresenter ifTrue:[ 
            aVDBPresenter instruction address = selectedInstructionBranchTargetAddress ifTrue:[ 
                ^ self styleSheet colorAt: 'instructionListApplication.branchTargetBackgroundColor' default:[ Color blue lighter lighter ]
            ].
        ].
        selectedInstructionBasicBlock notNil ifTrue:[ 
            | successor2 |

            successor2 := selectedInstructionBasicBlock successor2.
            (successor2 notNil and:[ successor2 includesAddress: aVDBPresenter address] ) ifTrue:[
                ^ self styleSheet colorAt: 'instructionListApplication.successor2BasicBlockBackgroundColor' default:[ Color blue lighter lighter lighter ]
            ].
        ].
    ].
    selectedInstructionBasicBlock notNil ifTrue:[ 
        (selectedInstructionBasicBlock includesAddress: aVDBPresenter address) ifTrue:[ 
            ^ self styleSheet colorAt: 'instructionListApplication.currentBasicBlockBackgroundColor' default:[ Color yellow lighter lighter ].
        ].
    ].

    "
    VDBInstructionListApplication flushDefaultStyleSheet; defaultStyleSheet
    "

    ^ nil

    "Created: / 26-06-2018 / 11:26:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 21-10-2019 / 01:10:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 29-10-2021 / 12:07:31 / Jan Vrany <jan.vrany@labware.com>"
!

disassemblableHolder
    disassemblableHolder isNil ifTrue:[
        disassemblableHolder := ValueHolder new.
        disassemblableHolder addDependent:self.
    ].
    ^ disassemblableHolder

    "Created: / 10-06-2019 / 10:03:38 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

disassemblableHolder:something
    "set the 'frameHolder' value holder (automatically generated)"

    |oldValue newValue|

    disassemblableHolder notNil ifTrue:[
        oldValue := disassemblableHolder value.
        disassemblableHolder removeDependent:self.
    ].
    disassemblableHolder := something.
    disassemblableHolder notNil ifTrue:[
        disassemblableHolder addDependent:self.
    ].
    newValue := disassemblableHolder value.
    oldValue ~~ newValue ifTrue:[
        self update:#value with:newValue from:disassemblableHolder.
    ].

    "Created: / 10-06-2019 / 10:03:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

foregroundColorFor: aVDBAbstractPresenter
    ^ aVDBAbstractPresenter isInstructionsAndSourcePresenter 
        ifTrue:[ Color gray ]
        ifFalse:[ nil ]

    "Created: / 26-06-2018 / 13:07:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

frameHolder
    "return/create the 'frameHolder' value holder (automatically generated)"

    frameHolder isNil ifTrue:[
        frameHolder := ValueHolder new.
        frameHolder addDependent:self.
    ].
    ^ frameHolder
!

frameHolder:something
    "set the 'frameHolder' value holder (automatically generated)"

    |oldValue newValue|

    frameHolder notNil ifTrue:[
        oldValue := frameHolder value.
        frameHolder removeDependent:self.
    ].
    frameHolder := something.
    frameHolder notNil ifTrue:[
        frameHolder addDependent:self.
    ].
    newValue := frameHolder value.
    oldValue ~~ newValue ifTrue:[
        self update:#value with:newValue from:frameHolder.
    ].
!

instructionListHolder
    "return/create the 'instructionListHolder' value holder (automatically generated)"

    instructionListHolder isNil ifTrue:[
        instructionListHolder := ValueHolder new.
        instructionListHolder addDependent:self.
    ].
    ^ instructionListHolder
!

instructionListHolder:valueModel
    "set the 'instructionListHolder' value holder (automatically generated)"

    |oldValue newValue|

    instructionListHolder notNil ifTrue:[
        oldValue := instructionListHolder value.
        instructionListHolder removeDependent:self.
    ].
    instructionListHolder := valueModel.
    instructionListHolder notNil ifTrue:[
        instructionListHolder addDependent:self.
    ].
    newValue := instructionListHolder value.
    oldValue ~~ newValue ifTrue:[
        self update:#value with:newValue from:instructionListHolder.
    ].
!

selectedInstructionHolder
    "return/create the 'selectedInstructionHolder' value holder (automatically generated)"

    selectedInstructionHolder isNil ifTrue:[
        selectedInstructionHolder := ValueHolder new.
        selectedInstructionHolder addDependent:self.
    ].
    ^ selectedInstructionHolder
!

selectedInstructionHolder:valueModel
    "set the 'selectedInstructionHolder' value holder (automatically generated)"

    |oldValue newValue|

    selectedInstructionHolder notNil ifTrue:[
        oldValue := selectedInstructionHolder value.
        selectedInstructionHolder removeDependent:self.
    ].
    selectedInstructionHolder := valueModel.
    selectedInstructionHolder notNil ifTrue:[
        selectedInstructionHolder addDependent:self.
    ].
    newValue := selectedInstructionHolder value.
    oldValue ~~ newValue ifTrue:[
        self update:#value with:newValue from:selectedInstructionHolder.
    ].
! !

!VDBInstructionListApplication methodsFor:'change & update'!

enqueueDelayedUpdateBasicBlocks
    self enqueueMessage: #delayedUpdateBasicBlocks

    "Created: / 26-06-2018 / 12:20:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 18-02-2019 / 10:23:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

enqueueDelayedUpdateButtonEnablements
    self enqueueMessage: #delayedUpdateButtonEnablements

    "Created: / 03-10-2018 / 12:12:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 18-02-2019 / 10:23:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

update:aspect with:param from:sender
    "Invoked when an object that I depend upon sends a change notification."

    sender == instructionListHolder ifTrue:[
        self enqueueDelayedUpdateInternalList.
        ^ self.
    ].
    sender == disassemblableHolder ifTrue:[ 
        self enqueueDelayedUpdateContents.
        ^ self.
    ].
    sender == frameHolder ifTrue:[ 
        self enqueueDelayedInvalidateInternalList.
        ^ self.
    ].
    sender == selectedInstructionHolder ifTrue:[ 
        self updateAfterSelectedInstructionChanged.
    ].
    super update:aspect with:param from:sender

    "Modified: / 23-01-2019 / 16:37:11 / jv"
    "Modified: / 10-06-2019 / 12:43:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

updateAfterSelectedInstructionChanged
    | selected |

    selected := self selectedInstructionHolder value.
    selected notNil ifTrue:[
        selected isBranch ifTrue:[ 
            selectedInstructionBranchTargetAddress := selected branchTarget.
        ] ifFalse:[ 
            selectedInstructionBranchTargetAddress := nil.
        ].
        instructionBasicBlocks size > 1 ifTrue:[
            selectedInstructionBasicBlock := instructionBasicBlocks detect:[:e | e includesAddress: selected address ] ifNone: [ nil ]
        ] ifFalse:[ 
            selectedInstructionBasicBlock := nil.
        ].
    ] ifFalse:[ 
        selectedInstructionBranchTargetAddress := nil.
        selectedInstructionBasicBlock := nil.

    ].

    self enqueueDelayedInvalidateInternalList.

    "Created: / 26-06-2018 / 11:33:06 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 03-07-2018 / 14:56:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

updateButtonEnablements
    self enqueueDelayedUpdateButtonEnablements

    "Created: / 01-09-2018 / 14:49:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 10-06-2019 / 10:11:50 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication methodsFor:'change & update-delayed'!

delayedInvalidateInternalList
    | frame |

    "/ First, force update of )cached) frames to get 
    "/ updated PC.
    frame := self frame.
    (frame notNil and:[frame thread isRunning not]) ifTrue:[
        GDBError catch: [ frame thread stack ]
    ].    
    super delayedInvalidateInternalList

    "Created: / 03-10-2018 / 12:10:01 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 12-06-2019 / 15:59:22 / jv"
    "Modified: / 12-06-2019 / 16:26:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

delayedUpdateBasicBlocks
    | disassemblable |

    disassemblable := self disassemblable.
    disassemblable notNil ifTrue: [
        instructionBasicBlocks := InstructionBasicBlocksCache at: disassemblable ifAbsent: nil.
        instructionBasicBlocks isNil ifTrue: [ instructionBasicBlocksAnalyzerJob restart ].
        self enqueueDelayedInvalidateInternalList.
    ].

    "Created: / 26-06-2018 / 12:20:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-08-2018 / 11:33:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-05-2022 / 13:54:05 / Jan Vrany <jan.vrany@labware.com>"
!

delayedUpdateButtonEnablements
    | frame thread threadIsStopped canReverse |

    frame := self frame.
    (frame notNil and:[frame respondsTo: #thread]) ifTrue:[ 
        thread := frame thread.
    ].
    threadIsStopped := thread notNil and:[ thread isStopped ].
    canReverse := debugger notNil and:[ debugger hasFeature: 'reverse' ].

    self canExecStepIntoInstructionHolder value: threadIsStopped.
    self canExecStepOverInstructionHolder value: threadIsStopped.

    self canExecBackIntoInstructionHolder value: threadIsStopped & canReverse.
    self canExecBackOverInstructionHolder value: threadIsStopped & canReverse.

    "Created: / 03-10-2018 / 12:12:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 21-10-2019 / 01:13:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

delayedUpdateContents
    | disassemblable frame instructions |

    disassemblable := self disassemblable.
    frame := self frame.

    "/ If there's no disassemblable nor frame, there's nothing
    "/ to update from. In this case, just show a bunch of instruction
    "/ someone may have set.
    (disassemblable isNil and: [ frame isNil ]) ifTrue: [ 
        self hideMessage.
        contentsValid := true.        
        ^ self.                       
    ].

    instructions := #().
    (disassemblable notNil "and:[ disassemblable hasSymbol ]") ifTrue:[
        [ 
            instructions := InstructionListCache at: disassemblable ifAbsentPut: [disassemblable disassemble].
            self hideMessage.
        ] on: GDBError do:[:ex | 
            self showMessage: (self resources string: (ex parameter propertyAt: #msg)).
        ].
    ].
    self instructionList: instructions.
    self updateButtonEnablements.

    "Created: / 01-10-2018 / 12:55:00 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 10-06-2019 / 10:50:34 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-05-2022 / 10:57:42 / Jan Vrany <jan.vrany@labware.com>"
!

delayedUpdateInternalList
    | address root |

    address := 0.
    root := self internalListHolder root.
    root children:
        ((self instructionListHolder value ? #()) collect:[:model | 
            (model isKindOf: GDBInstructionsAndSourceLine) ifTrue:[
                (VDBInstructionsAndSourcePresenter new) setInstructionsAndSource: model;
                    parent:root;
                    yourself
            ] ifFalse:[
                | instruction |

                instruction := model.
                instruction address isNil ifTrue: [ 
                    "/ If the instruction has no address (for example, when VDBInstructionList
                    "/ is used out of VDB for displaying code buffer, then compute an address
                    "/ as offset from the beggining.
                    instruction propertyAt: #address put: address.
                    address := address + instruction opcodes size.
                ] ifFalse: [ 
                    address := instruction address.
                ].

                (VDBInstructionPresenter new) setInstruction: instruction;
                    parent:root;
                    yourself
            ].
        ]).
    root expand.
    root children do:[:each | each expand ].
    self frame notNil ifTrue:[ 
        self scrollToAddress: self frame address.
    ].
    internalListView notNil ifTrue:[
        internalListView invalidate.
    ].
    self enqueueDelayedUpdateBasicBlocks.

    "Created: / 27-02-2015 / 15:47:18 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 10-06-2019 / 12:45:55 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 29-10-2021 / 11:40:26 / Jan Vrany <jan.vrany@labware.com>"
!

delayedUpdateSelection
    | internalSelection |

    internalSelection := self internalSelectionHolder value.
    internalSelection notNil ifTrue:[
        self selectedInstructionHolder value: internalSelection instruction
    ] ifFalse:[ 
        self selectedInstructionHolder value: nil
    ].

    "Modified: / 22-06-2018 / 15:08:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication methodsFor:'event handling'!

onBreakpointEvent: aGDBBreakpointEvent

    "/ The list shows breakpoints, if a breakpoint is set on
    "/ shown instruction. Therefore we need to redraw the list
    "/ when breakpoint is added/removed or modified (disabled/enabled)
    self enqueueDelayedInvalidateInternalList.

    "Created: / 07-01-2022 / 20:19:21 / Jan Vrany <jan.vrany@labware.com>"
!

onCommandResultEvent: aGDBStoppedEvent
    self updateButtonEnablements

    "Created: / 01-06-2017 / 23:43:00 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

onMemoryChangedEvent: aGDBMemoryChangedEvent
    | instructions |

    "/ Flush caches (maybe not needed, but just ot be on the
    "/ safe side)
    InstructionListCache removeAll.
    InstructionBasicBlocksCache removeAll.

    instructions := self instructionList.
    instructions notEmptyOrNil ifTrue: [ 
        | instructionRange memoryRange |

        instructionRange := instructions first address to: instructions last address.
        memoryRange := aGDBMemoryChangedEvent addr to: aGDBMemoryChangedEvent addr + aGDBMemoryChangedEvent len.

        (instructionRange intersect: memoryRange) notEmpty ifTrue: [ 
            self enqueueDelayedUpdateContents.
        ].
    ].

    "Created: / 18-11-2021 / 23:25:51 / Jan Vrany <jan.vrany@labware.com>"
    "Modified: / 26-05-2022 / 11:03:11 / Jan Vrany <jan.vrany@labware.com>"
!

onRunningEvent: aGDBStoppedEvent
    self enqueueDelayedUpdateButtonEnablements

    "Created: / 21-09-2014 / 22:44:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 03-10-2018 / 12:12:55 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

onStoppedEvent: aGDBStoppedEvent
    "/ Flush caches (maybe not needed, but just ot be on the
    "/ safe side)
    InstructionListCache removeAll.
    InstructionBasicBlocksCache removeAll.  

    self frame notNil ifTrue:[
        self enqueueDelayedInvalidateInternalList.
        self enqueueDelayedUpdateButtonEnablements.
    ].

    "Created: / 06-08-2018 / 14:45:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 03-10-2018 / 12:13:21 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-05-2022 / 11:03:32 / Jan Vrany <jan.vrany@labware.com>"
! !

!VDBInstructionListApplication methodsFor:'hooks'!

postBuildInternalListView: aView
    super postBuildInternalListView: aView.
    internalListView addDependent: self.

    "Created: / 23-01-2019 / 16:10:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication methodsFor:'initialization & release'!

initialize
    super initialize.
    instructionBasicBlocksAnalyzerJob := 
            BackgroundJob named: 'VDB basic block analyzer' 
                                                      on:[self  instructionsAnalyze].

    "Created: / 26-05-2022 / 12:41:53 / Jan Vrany <jan.vrany@labware.com>"
!

release
    super release.
    internalListView notNil ifTrue:[ 
        internalListView removeDependent: self.
    ].

    "Created: / 23-01-2019 / 16:11:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

subscribe   
    super subscribe.

    debugger announcer 
        when: GDBStoppedEvent       send: #onStoppedEvent:  to: self;
        when: GDBRunningEvent       send: #onRunningEvent:  to: self;
        when: GDBMemoryChangedEvent send: #onMemoryChangedEvent:  to: self;
        "/when: GDBExitEvent          send: #onExitEvent:     to: self;
        when: GDBCommandResultEvent send: #onCommandResultEvent: to: self;
        when: GDBBreakpointEvent    send: #onBreakpointEvent: to: self.

    contentsValid := false.

    "Created: / 06-08-2018 / 14:44:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 23-01-2019 / 14:29:29 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 07-01-2022 / 20:20:01 / Jan Vrany <jan.vrany@labware.com>"
! !

!VDBInstructionListApplication methodsFor:'menu actions'!

doShowBranchTarget
    | selection |

   selection := self selectedInstructionHolder value.
   (selection notNil and:[selection isBranch]) ifTrue:[ 
        | branchTarget |

        branchTarget := selection branchTarget.
        "/ For indirect jumps, `branchTarget` is `nil`.
        branchTarget notNil ifTrue:[ 
            self doShowInstructionAtAddress: branchTarget  
        ].
   ].

    "Modified: / 01-09-2018 / 14:32:39 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doShowCurrent
    | frame |

    frame := self frameHolder value.
    frame notNil ifTrue:[ 
        self doShowInstructionAtAddress: frame address  
    ].

    "Modified: / 01-09-2018 / 14:27:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doShowInstructionAtAddress: address
    self scrollToAddress: address

    "Created: / 01-09-2018 / 14:26:54 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doShowSelection
   | selection |

   selection := self selectedInstructionHolder value.
   selection notNil ifTrue:[ 
       self doShowInstructionAtAddress: selection address
   ].

    "Modified: / 01-09-2018 / 14:32:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication methodsFor:'menu actions-exec'!

doExec: command
    debugger send: command andWait: false.
    self enqueueDelayedInvalidateInternalList

    "Created: / 21-09-2014 / 21:50:19 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-03-2018 / 17:57:15 / jv"
    "Modified: / 01-09-2018 / 22:33:06 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doExecBackIntoInstruction
    self doExec:(GDBMI_exec_step_instruction arguments:#('--reverse'))

    "Modified: / 01-09-2018 / 22:16:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doExecBackOverInstruction
    self doExec:(GDBMI_exec_next_instruction arguments:#('--reverse'))

    "Modified: / 01-09-2018 / 22:16:01 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doExecStepIntoInstruction
    self doExec:GDBMI_exec_step_instruction new

    "Created: / 17-11-2020 / 19:38:43 / Jan Vrany <jan.vrany@labware.com>"
!

doExecStepOverInstruction
    self doExec:GDBMI_exec_next_instruction new

    "Modified: / 01-09-2018 / 21:59:50 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication methodsFor:'menu aspects'!

canShowBranchTargetHolder
    ^ BlockValue 
        with:[ :selection | selection value notNil and:[selection value isBranch and:[selection value branchTarget notNil]]] 
        argument: self selectedInstructionHolder

    "Modified: / 01-09-2018 / 15:32:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

canShowCurrentHolder
    ^ BlockValue 
        with:[ :frame | frame value notNil and:[frame value address notNil ]]
        argument: self frameHolder

    "Modified: / 01-09-2018 / 15:33:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

canShowSelectionHolder
    ^ BlockValue 
        with:[ :selection | selection value notNil ] 
        argument: self selectedInstructionHolder

    "Modified: / 01-09-2018 / 15:33:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication methodsFor:'menu aspects-exec'!

canExecBackIntoInstructionHolder
    canExecBackIntoInstructionHolder isNil ifTrue:[
        canExecBackIntoInstructionHolder := ValueHolder with: false.
    ].
    ^ canExecBackIntoInstructionHolder

    "Modified: / 01-09-2018 / 22:07:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

canExecBackOverInstructionHolder
    canExecBackOverInstructionHolder isNil ifTrue:[
        canExecBackOverInstructionHolder := ValueHolder with: false.
    ].
    ^ canExecBackOverInstructionHolder

    "Modified: / 01-09-2018 / 22:07:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

canExecStepIntoInstructionHolder
    canExecStepOverInstructionHolder isNil ifTrue:[
        canExecStepOverInstructionHolder := ValueHolder with: false.
    ].
    ^ canExecStepOverInstructionHolder

    "Modified: / 01-09-2018 / 22:08:24 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

canExecStepOverInstructionHolder
    canExecStepOverInstructionHolder isNil ifTrue:[
        canExecStepOverInstructionHolder := ValueHolder with: false.
    ].
    ^ canExecStepOverInstructionHolder

    "Modified: / 01-09-2018 / 22:08:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication methodsFor:'private'!

instructionPresentersDo: aBlock
    self internalListHolder root children do:[:each |
        each isInstructionPresenter ifTrue:[ 
            aBlock value: each
        ] ifFalse:[ 
            each children do: aBlock
        ].
    ]

    "Created: / 26-06-2018 / 12:30:57 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

instructionsAnalyze
    "Analyze instructions (if any) and build basic blocks"
    | disassemblable instructions |

    disassemblable := self disassemblable.
    instructions := OrderedCollection new.
    self instructionsDo: [ :e | instructions add: e ].
    instructionBasicBlocks := VDBInstructionBasicBlock analyze: instructions.
    disassemblable == self disassemblable ifTrue: [
        InstructionBasicBlocksCache at:  disassemblable put:  instructionBasicBlocks.
    ].
    self enqueueDelayedInvalidateInternalList.

    "Created: / 26-05-2022 / 12:41:34 / Jan Vrany <jan.vrany@labware.com>"
    "Modified: / 26-05-2022 / 14:05:42 / Jan Vrany <jan.vrany@labware.com>"
!

instructionsDo: aBlock
    self instructionListHolder value ? #() do:[:each |
        each instructionsDo: aBlock
    ]

    "Created: / 16-08-2018 / 11:32:41 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 29-10-2021 / 11:40:30 / Jan Vrany <jan.vrany@labware.com>"
! !

!VDBInstructionListApplication methodsFor:'queries'!

canSelect: anItem
    ^ anItem isInstructionPresenter

    "Created: / 22-06-2018 / 15:07:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication methodsFor:'scrolling'!

scrollToAddress: address
    internalListView isNil ifTrue:[ ^ self ].
    (self disassemblable notNil and:[ self instructionList notEmptyOrNil ]) ifTrue:[ 
        self instructionPresentersDo:[ :each |
            each address = address ifTrue:[ 
                self scrollToListItem: each.
            ].
        ]
    ].

    "Created: / 07-08-2018 / 13:36:27 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 10-06-2019 / 10:16:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBInstructionListApplication class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !


VDBInstructionListApplication initialize!
