#
# jv:vdb - Visual / VM Debugger
# Copyright (C) 2015, 2022 Jan Vrany
#
# This software is licensed under 'Creative Commons Attribution-NonCommercial 4.0 International License'
#
# You may find a full license text in LICENSE.txt or at http://creativecommons.org/licenses/by-nc/4.0/
#

"""
A set of handy utilities to simplify writing Python scripts
using C-like constructs. Useful when transcribing come C code
to Python for use in GDB (for traversing data structures and so on)
"""


import gdb

int8_t = gdb.selected_inferior().architecture().integer_type(8)
int16_t = gdb.selected_inferior().architecture().integer_type(16)
int32_t = gdb.selected_inferior().architecture().integer_type(32)
int64_t = gdb.selected_inferior().architecture().integer_type(64)

uint8_t = gdb.selected_inferior().architecture().integer_type(8, False)
uint16_t = gdb.selected_inferior().architecture().integer_type(16, False)
uint32_t = gdb.selected_inferior().architecture().integer_type(32, False)
uint64_t = gdb.selected_inferior().architecture().integer_type(64, False)

def sizeof(typeOrName):
    if isinstance(typeOrName, gdb.Type):
        typ = typeOrName
    elif isinstance(typeOrName, str):
        typ = gdb.lookup_type(typeOrName)
    else:
        raise ValueError('Invalid parameter (not a str or gdb.Type)')
    return typ.sizeof

